﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;


public class DotToImageConverter {
  private const string EXT_IMG = "png";

  public static Image CreateImageFrom(string dotSource) {
    Process dotProc = StartDotProcess();

    StreamWriter writer = dotProc.StandardInput;
    writer.Write(dotSource);
    writer.Close();

    Stream imageStream = dotProc.StandardOutput.BaseStream;
    Image image = Image.FromStream(imageStream);

    dotProc.WaitForExit();

    return image;
  }

  public static byte[] CreateByteArrayFrom(string dotSource) {
    Process dotProc = StartDotProcess();

    StreamWriter writer = dotProc.StandardInput;
    writer.Write(dotSource);
    writer.Close();

    Stream imageStream = dotProc.StandardOutput.BaseStream;
    byte[] image = GetByteArray(imageStream);

    dotProc.WaitForExit();
    return image;

  }

  private static Process StartDotProcess() {
    string parameters = string.Format("-T{0}", EXT_IMG);
    var info1 = new ProcessStartInfo("dot.exe", parameters);

    info1.CreateNoWindow = true;
    info1.UseShellExecute = false;
    info1.RedirectStandardInput = true;
    info1.RedirectStandardOutput = true;
    ProcessStartInfo info = info1;
    return Process.Start(info);
  }

  private static byte[] GetByteArray(Stream imageStream) {
    using (MemoryStream ms = new MemoryStream()) {
      byte[] buffer = new byte[32768];
      while (true) {
        int read = imageStream.Read(buffer, 0, buffer.Length);
        if (read < 1)
          break;
        ms.Write(buffer, 0, read);
      }
      return ms.ToArray();
    }
  }

}