using System.Collections.Generic;

namespace Plux.Framework {
  public enum AutoReleaseMode {
    Off,
    On,
    SlotDefined
  }

  public class ReplacePlug {
    private AutoReleaseMode autoRelease = AutoReleaseMode.SlotDefined;
    public bool Global;
    private bool slotDefinedAutoRelease;
    public SlotInfo slotInfo;
    public ReplacePlug() {}
    public ReplacePlug(SlotInfo slotInfo) {
      Register(slotInfo);
    }
    public ReplacePlug(SlotInfo slotInfo, bool global) : this(slotInfo) {
      Global = global;
    }
    public SlotInfo SlotInfo {
      get { return slotInfo; }
      set {
        if(value == slotInfo) return;
        Unregister(slotInfo);
        Register(value);
      }
    }
    public AutoReleaseMode AutoRelease {
      get { return autoRelease; }
      set {
        autoRelease = value;
        switch(value) {
          case AutoReleaseMode.Off:
            slotInfo.AutoRelease = false;
            break;
          case AutoReleaseMode.On:
            slotInfo.AutoRelease = true;
            break;
          case AutoReleaseMode.SlotDefined:
            slotInfo.AutoRelease = slotDefinedAutoRelease;
            break;
        }
      }
    }

    private void Register(SlotInfo si) {
      if(si == null) return;
      slotInfo = si;
      slotDefinedAutoRelease = si.AutoRelease;
      si.Plugging += Plugging;
      si.Deregistering += Deregistering;
    }
    private void Unregister(SlotInfo si) {
      if(si == null) return;
      si.Plugging -= Plugging;
      si.Deregistering -= Deregistering;
      si.AutoRelease = slotDefinedAutoRelease;
    }

    private void Deregistering(object sender, CancelRegisterEventArgs args) {
      ExtensionTypeInfo type = args.PlugTypeInfo.ExtensionTypeInfo;
      for(int i = type.ExtensionInfos.Count - 1; i >= 0; --i) {
        ExtensionInfo ext = type.ExtensionInfos[i];
        if(ext.Owner != slotInfo) continue;
        int plugged = 0;
        foreach(PlugInfo plug in ext.PlugInfos)
          plugged += plug.PluggedInSlots.Count;
        if(plugged == 1) ext.Release();
      }
    }
    public void Plugging(object sender, CancelPlugEventArgs args) {
      if(Global) {
        SlotInfo[] slots = Runtime.Repository.SlotInfos.Find(slotInfo.Name);
        foreach(SlotInfo slot in slots)
          foreach(PlugInfo pi in slot.PluggedPlugInfos)
            if(pi != args.PlugInfo) slot.Unplug(pi);
        args.PlugInfo.Plug();
      }
      else
        foreach (PlugInfo pi in slotInfo.PluggedPlugInfos)
          if (pi != args.PlugInfo) slotInfo.Unplug(pi);
    }
  }

  internal class ReplacePlugStartup : SlotStartup<ReplaceSlotAttribute> {
    private static readonly Dictionary<SlotInfo, ReplacePlug> data =
      new Dictionary<SlotInfo, ReplacePlug>();
    public override void Register(ExtensionTypeInfo extType) {}
    public override void Unregister(ExtensionTypeInfo extType) {}
    public override void SlotCreated(SlotInfo slot, ReplaceSlotAttribute attr) {
      if(attr.Plug)
        data[slot] = new ReplacePlug(slot, attr.Global);
    }
    public override void SlotReleased(SlotInfo slot, ReplaceSlotAttribute attr) {
      if(attr.Plug)
        data.Remove(slot);
    }
  }
}